package com.xhkjedu.sexam.controller.paper;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.EPaperCorrectquestion;
import com.xhkjedu.sexam.model.paperstudent.EPaperStudentQuestion;
import com.xhkjedu.sexam.service.paper.EPaperCorrectTeacherService;
import com.xhkjedu.sexam.vo.paper.EPTeacherVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname EPaperCorrectTeacherController
 * @description 教师阅卷
 * @date 2022/7/27 11:24
 **/
@RestController
@RequestMapping("/ept")
public class EPaperCorrectTeacherController {
    @Resource
    private EPaperCorrectTeacherService correctTeacherService;

    /**
     * 待阅卷列表
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/27 14:07
     **/
    @PostMapping("/list_pending")
    public ResultVo listPending(@RequestBody ESubject vo) {
        Integer teacherid = vo.getTeacherid();
        N_Utils.validation(new Object[]{teacherid, "教师id", 1});
        List<Map> list = correctTeacherService.listPending(teacherid);
        return new ResultVo(0, "获取成功", list);
    }

    /**
     * 已阅卷列表
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/27 14:18
     **/
    @PostMapping("/list_marked")
    public ResultVo listMarked(@RequestBody ESubject vo) {
        Integer teacherid = vo.getTeacherid();
        Integer page = vo.getPage();
        Integer pageSize = vo.getPageSize();
        N_Utils.validation(new Object[]{teacherid, "教师id", 1, page, "页码", 1, pageSize, "显示条数", 1});
        PageHelper.startPage(page, pageSize);
        List<Map> list = correctTeacherService.listMarked(teacherid);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0, "获取成功", pageResult);
    }

    /**
     * 随机获取待批试题
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/27 16:21
     **/
    @PostMapping("/get_cq")
    public ResultVo getCorrectQuestion(@RequestBody EPTeacherVo vo) {
        Integer epid = vo.getEpid();
        Integer teacherid = vo.getTeacherid();
        String qorders = vo.getQorders();
        N_Utils.validation(new Object[]{epid, "试卷id", 1, teacherid, "教师id", 1, qorders, "页码", 2});
        return correctTeacherService.getCorrectQuestion(epid, teacherid, qorders, vo.getClassids());
    }

    /**
     * 获取上一份批阅试题
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/27 16:33
     **/
    @PostMapping("/get_pre")
    public ResultVo getPreQuestion(@RequestBody EPTeacherVo vo) {
        Integer epid = vo.getEpid();
        Integer teacherid = vo.getTeacherid();
        N_Utils.validation(new Object[]{epid, "试卷id", 1, teacherid, "教师id", 1});
        return correctTeacherService.getPreQuestion(epid, teacherid);
    }

    /**
     * 阅卷记录
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/27 16:41
     **/
    @PostMapping("/list_jl")
    public ResultVo listRecord(@RequestBody EPaperCorrectquestion vo) {
        Integer epid = vo.getEpid();
        Integer teacherid = vo.getTeacherid();
        Integer qorder = vo.getQorder();
        Integer page = vo.getPage();
        Integer pageSize = vo.getPageSize();
        N_Utils.validation(new Object[]{epid, "试卷id", 1, teacherid, "教师id", 1, qorder, "试题排序", 1
                , page, "页码", 1, pageSize, "显示条数", 1});
        PageHelper.startPage(page, pageSize);
        List<Map> list = correctTeacherService.listRecord(epid, teacherid, qorder);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0, "获取成功", pageResult);
    }

    /**
     * 批阅试题
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/28 15:30
     **/
    @PostMapping("/correct")
    public ResultVo correctQuestion(@RequestBody EPaperStudentQuestion vo) {
        Integer epsid = vo.getEpsid();
        Integer epsqid = vo.getEpsqid();
        Integer checkid = vo.getCheckid();
        N_Utils.validation(new Object[]{epsid, "试卷学生id", 1, epsqid, "试卷学生试题id", 1, checkid, "批阅人", 1});
        return correctTeacherService.correctQuestion(vo);
    }
}
