package com.xhkjedu.sexam.controller.paperstudent;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.sexam.model.paperstudent.EPaperStudent;
import com.xhkjedu.sexam.model.paperstudent.EPaperStudentQuestion;
import com.xhkjedu.sexam.service.paperstudent.EPaperStudentService;
import com.xhkjedu.sexam.vo.paperstudent.EPaperStudentVo;
import com.xhkjedu.sexam.vo.paperstudent.PaperStudentWebVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description 学生考试
 * @Author WN
 * Date 2022/7/26 13:34
 **/
@RestController
@RequestMapping("/eps")
@Slf4j
public class EPaperStudentController {
    @Resource
    private EPaperStudentService ePaperStudentService;

    //学生考试列表--进行中
    @PostMapping("/jxlist")
    public ResultVo listExamPapersForStudentJx(@RequestBody EPaperStudent eps) {
        N_Utils.validation(new Object[]{eps.getStudentid(), "学生id", 1});
        eps.setSstate(1); //进行中

        List<Map> rtnlst = ePaperStudentService.listExamPapersForStudent(eps);
        return new ResultVo(0, "成功获取考试列表", rtnlst);
    }

    //学生考试列表--已结束
    @PostMapping("/jslist")
    public ResultVo listExamPapersForStudentJs(@RequestBody EPaperStudent eps) {
        N_Utils.validation(new Object[]{eps.getStudentid(), "学生id", 1});
        eps.setSstate(2);//结束的考试
        Integer page = eps.getPage();
        Integer pageSize = eps.getPageSize();
        N_Utils.validation(new Object[]{page, "显示页码", 1, pageSize, "显示条数", 1});
        PageHelper.startPage(page, pageSize);
        List<Map> rtnlst = ePaperStudentService.listExamsEndForStudent(eps);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(rtnlst));
        return new ResultVo(0, "成功获取考试列表", pageResult);
    }

    //待考界面--考试信息
    @PostMapping("/info")
    public ResultVo getExamSubjectPaper(@RequestBody EPaperStudent eps) {
        N_Utils.validation(new Object[]{eps.getEpsid(), "考试学生id", 1});
        Map map = ePaperStudentService.getExamSubjectPaperByEpsid(eps.getEpsid());
        return new ResultVo(0, "成功获取考试信息", map);
    }

    //学生开始考试(android)
    @PostMapping("/spa")
    public ResultVo startPaperForAndroid(@RequestBody EPaperStudent eps) {
        N_Utils.validation(new Object[]{eps.getEpsid(), "学生试卷id", 1});
        List<Map> list = ePaperStudentService.startPaperForAndroid(eps);
        return new ResultVo(0, "成功获取学生试卷", list);
    }

    //学生开始考试(web)
    @PostMapping("/spw")
    public ResultVo startPaperForWeb(@RequestBody EPaperStudent eps) {
        N_Utils.validation(new Object[]{eps.getEpsid(), "学生试卷id", 1});
        EPaperStudentVo psvo = ePaperStudentService.startPaperForWeb(eps);
        return new ResultVo(0, "成功获取学生试卷", psvo);
    }

    //提交单题
    @PostMapping("/scq")
    public ResultVo saveCommitQuestion(@RequestBody EPaperStudentQuestion sq) {
        try {
            N_Utils.validation(new Object[]{sq.getEpsqid(), "试题id", 1});
            ePaperStudentService.saveCommitQuestion(sq);
            return new ResultVo(0, "成功提交试题");
        } catch (Exception e) {
            return new ResultVo(1, "提交试题失败");
        }
    }

    //提交试卷
    @PostMapping("/scp")
    public ResultVo saveCommitPaper(@RequestBody EPaperStudent ps) {
        try {
            N_Utils.validation(new Object[]{ps.getEpsid(), "试卷id", 1});
            ePaperStudentService.saveCommitPaper(ps.getEpsid());
            return new ResultVo(0, "成功提交试卷");
        } catch (Exception e) {
            log.error("提交试卷失败：" + e.getMessage());
            return new ResultVo(1, "提交失败");
        }
    }

    //提交试卷--网页端
    @PostMapping("/scpw")
    public ResultVo saveCommitPaperWeb(@RequestBody PaperStudentWebVo ps) {
        try {
            N_Utils.validation(new Object[]{ps.getEpsid(), "试卷id", 1});
            List<EPaperStudentQuestion> qlist = ps.getQuestions();
            if (N_Utils.isListEmpty(qlist)) {
                return new ResultVo(1, "试题不能为空");
            }
            ePaperStudentService.saveCommitPaperForWeb(ps);
            return new ResultVo(0, "成功提交试卷");
        } catch (Exception e) {
            log.error("提交试卷(web)失败：" + e.getMessage());
            return new ResultVo(1, "提交失败");
        }
    }

    /**
     * @Description 保存扫描的答题卡
     * @Param [ps]
     * @Return com.xhkjedu.vo.ResultVo
     * @Author wn
     * @Date 2022/8/3 14:35
     **/
    @PostMapping("/sca")
    public ResultVo saveCommitPaperScantron(@RequestBody PaperStudentWebVo ps) {
        try {
            N_Utils.validation(new Object[]{ps.getEpid(), "试卷id", 1,ps.getExamno(),"考号",2});
            ePaperStudentService.saveCommitPaperForScantron(ps);
            return new ResultVo(0, "成功保存学生答题卡");
        } catch (Exception e) {
            log.error("保存学生答题卡失败：" + e.getMessage());
            return new ResultVo(1, "保存学生答题卡失败");
        }
    }


}
