package com.xhkjedu.sexam.service.paperstudent;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.paper.EPaperQtypeMapper;
import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentMapper;
import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentQuestionMapper;
import com.xhkjedu.sexam.mapper.system.UserMapper;
import com.xhkjedu.sexam.model.paper.EPaperQtype;
import com.xhkjedu.sexam.model.paper.EPaperQtypeQuestion;
import com.xhkjedu.sexam.model.paperstudent.EPaperStudent;
import com.xhkjedu.sexam.model.paperstudent.EPaperStudentQuestion;
import com.xhkjedu.sexam.utils.ConvertUtil;
import com.xhkjedu.sexam.utils.ExamUtil;
import com.xhkjedu.sexam.vo.paperstudent.EPaperStudentVo;
import com.xhkjedu.sexam.vo.paperstudent.PaperStudentWebVo;
import com.xhkjedu.sexam.vo.paperstudent.PsqAnswerVo;
import com.xhkjedu.utils.N_Utils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description 学生试卷信息
 * @Author WN
 * Date 2022/7/26 14:00
 **/
@Service
@Slf4j
public class EPaperStudentService {
    @Resource
    private EPaperStudentMapper ePaperStudentMapper;
    @Resource
    private EPaperQtypeMapper ePaperQtypeMapper;
    @Resource
    private EPaperStudentQuestionMapper ePaperStudentQuestionMapper;
    @Resource
    private UserMapper userMapper;
    @Autowired
    private ConvertUtil convertUtil;

    /**
     * @Description 学生进行中考试列表
     * @Param [eps]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/7/27 10:35
     **/
    public List<Map> listExamPapersForStudent(EPaperStudent eps) {
        return ePaperStudentMapper.listExamPapersForStudent(eps.getStudentid());
    }

    public Map getExamSubjectPaperByEpsid(Integer epsid){
        Map map = ePaperStudentMapper.getExamSubjectPaperByEpsid(epsid);
        Long times = null;
        if(map.get("sdate")!=null && map.get("begintime")!=null && map.get("endtime")!=null){
            long bg = ExamUtil.strToTimestamp(map.get("sdate").toString() + " " + map.get("begintime").toString() + ":00");
            long ed = ExamUtil.strToTimestamp(map.get("sdate").toString() + " " + map.get("endtime").toString() + ":00");
            times = ExamUtil.subtract(ed,bg);
        }
        map.put("times",times);
        return map;
    }

    /**
     * 学生已结束考试列表
     *
     * @Description
     * @Param [eps]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/7/27 11:34
     **/
    public List<Map> listExamsEndForStudent(EPaperStudent eps) {
        return ePaperStudentMapper.listExamsEndForStudent(eps.getStudentid());
    }

    /**
     * @Description 开始考试(andorid)
     * @Param [eps]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/27 11:48
     **/
    public List<Map> startPaperForAndroid(EPaperStudent eps) {
        Integer epsid = eps.getEpsid();
        //获取试卷基本信息
        EPaperStudentVo psvo = ePaperStudentMapper.getExamSubjectByEpsid(epsid);
        Integer ptype = psvo.getPtype();

        List<Map> questions = null;
        if (ptype == 1) {
            //题库试卷试题
            questions = ePaperStudentMapper.listStuPaperQuestionsForAndroid(epsid);
        } else {
            questions = ePaperStudentMapper.listStuPaperFjQuestionsForAndroid(epsid);
        }
        //判断是否生成学生试卷，未生成则需要生成，如果已生成直接返回
        if (questions == null || questions.size() == 0) {

            //保存试卷关联信息
            savePaperStudentQuestion(psvo);
            //保存后重新获取返回
            if (ptype == 1) {
                questions = ePaperStudentMapper.listStuPaperQuestionsForAndroid(epsid);
            } else {
                questions = ePaperStudentMapper.listStuPaperFjQuestionsForAndroid(epsid);
            }
        }
        return questions;
    }

    //保存学生试卷关联信息
    private void savePaperStudentQuestion(EPaperStudentVo psvo) {
        //生成学生试卷
        List<EPaperQtypeQuestion> pqlist = ePaperQtypeMapper.listQuestionsByEpid(psvo.getEpid());

        List<EPaperStudentQuestion> psqlist = new ArrayList<>();
        for (EPaperQtypeQuestion q : pqlist) {
            EPaperStudentQuestion sq = new EPaperStudentQuestion();
            sq.setClassid(psvo.getClassid());
            sq.setStudentid(psvo.getStudentid());
            sq.setEpid(psvo.getEpid());
            sq.setEpsid(psvo.getEpsid());
            sq.setEptqid(q.getEptqid());
            sq.setQuestionid(q.getQuestionid());
            sq.setQn(q.getQn());
            sq.setQorder(q.getQorder());
            sq.setQscore(q.getScore());
            sq.setAnswered(0);
            sq.setConverted(0);
            sq.setChecked(0);
            sq.setStuscore(0.0);
            sq.setCosttime(0);
            psqlist.add(sq);
        }
        ePaperStudentQuestionMapper.insertList(psqlist);//保存试卷
    }

    /**
     * @Description 开始考试（web）
     * @Param [eps]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/7/27 15:36
     **/
    public EPaperStudentVo startPaperForWeb(EPaperStudent eps) {
        Integer epsid = eps.getEpsid();
        //获取试卷基本信息
        EPaperStudentVo psvo = ePaperStudentMapper.getExamSubjectByEpsid(epsid);
        Integer ptype = psvo.getPtype();

        List<EPaperQtype> questions = null;
        if (ptype == 1) {
            //题库试卷试题
            questions = ePaperStudentMapper.listStuPaperQuestionsForWeb(epsid);
        } else {
            questions = ePaperStudentMapper.listStuPaperFjQuestionsForWeb(epsid);
        }
        //判断是否生成学生试卷，未生成则需要生成，如果已生成直接返回
        if (questions == null || questions.size() == 0) {
            //保存试卷关联信息
            savePaperStudentQuestion(psvo);
            //保存后重新获取返回
            if (ptype == 1) {
                questions = ePaperStudentMapper.listStuPaperQuestionsForWeb(epsid);
            } else {
                questions = ePaperStudentMapper.listStuPaperFjQuestionsForWeb(epsid);
            }
        }

        psvo.setQtypelist(questions);
        return psvo;
    }

    /**
     * @Description 提交单题
     * @Param [psq]
     * @Return void
     * @Author wn
     * @Date 2022/7/28 13:47
     **/
    public void saveCommitQuestion(EPaperStudentQuestion psq) throws Exception {
        try {
            //学生作答提交时先判断试卷是否设置的正确答案
            PsqAnswerVo answerVo = ePaperStudentQuestionMapper.getPaperQuestionAnswer(psq.getEpsqid());
            setCommitQuestion(psq,answerVo);
            ePaperStudentQuestionMapper.updateStuAnswer(psq);
        }catch (Exception e){
            log.error("提交试题失败："+e.getMessage());
            throw new Exception("提交试题失败");
        }

    }

    //判断单道题情况
    private void setCommitQuestion(EPaperStudentQuestion psq,PsqAnswerVo answerVo){
        int timestamp = N_Utils.getSecondTimestamp();
        psq.setAnswertime(timestamp);
        int coverted = 1;//是否转换0未转换1已转换2转换失败
        String stuanswer = psq.getStuanswer();
        List<String> stuanswers = JSON.parseArray(stuanswer, String.class);
        Double stuscore = 0D;

        if(N_Utils.isListEmpty(stuanswers) && N_Utils.isEmpty(psq.getStuanswertxt())
                && N_Utils.isEmpty(psq.getStuanswertxt())){
            psq.setStuscore(stuscore);
            psq.setChecked(2);
            psq.setChecktime(timestamp);
            psq.setAnswered(0);
        }else{
            //试卷已经设置了正确答案
            if (answerVo.getAnswered() == 1){
                String qanswer = answerVo.getAnswer();
                List<String> qanswers = JSON.parseArray(qanswer,String.class);
                if(N_Utils.isListEmpty(stuanswers) && N_Utils.isEmpty(psq.getStuanswertxt())
                        && N_Utils.isEmpty(psq.getStuanswertxt())){
                    psq.setStuscore(stuscore);
                    psq.setChecked(1);
                    psq.setChecktime(timestamp);
                    psq.setAnswered(0);
                }else{
                    int ctype = answerVo.getCtype();
                    if(N_Utils.isObjectiveQuestion(ctype)){
                        //客观题进行批阅
                        if(ctype == 1 || ctype == 4 || ctype == 5 || ctype == 6){
                            //单选、判断
                            if(stuanswer.equalsIgnoreCase(qanswer)) {
                                stuscore = answerVo.getScore();
                            }
                        }else if(ctype == 2){//多选
                            //多选、选错不得分
                            if(stuanswers.get(0).length() <= qanswer.length()){
                                String[] qanswerList = qanswers.get(0).split("");
                                String[] stuanswerList = stuanswers.get(0).split("");
                                int rightNum = 0;//选对个数
                                for(String sa : stuanswerList){
                                    int right = 0;
                                    for(String qa : qanswerList){
                                        if(qa.equals(sa)){
                                            right = 1;
                                            rightNum ++;
                                        }
                                    }
                                    if(right == 0){//说明选错
                                        rightNum = 0;
                                        break;
                                    }
                                }
                                if(rightNum>0){
                                    if(qanswerList.length == rightNum){
                                        //全选对
                                        stuscore = answerVo.getScore();
                                    }else if (qanswerList.length > rightNum){
                                        //少选得半分
                                        stuscore = ExamUtil.div(answerVo.getScore(),2,1);
                                    }
                                }
                            }
                        }else if(ctype == 7 || ctype == 8 || ctype == 10){
                            //完形填空、阅读理解、任务型阅读
                            Double qscore = ExamUtil.div(answerVo.getScore(),qanswers.size());
                            Double stuScoreDouble = 0d;
                            for(int i=0;i<qanswers.size();i++){
                                if(qanswers.get(i).equalsIgnoreCase(stuanswers.get(i))){
                                    stuScoreDouble = ExamUtil.add(stuScoreDouble,qscore);
                                }
                            }
                            stuscore = ExamUtil.round(stuScoreDouble,1);
                        }
                        if(stuscore.compareTo(answerVo.getScore()) == 1){
                            stuscore = answerVo.getScore();
                        }
                        psq.setStuscore(stuscore);
                        psq.setChecked(2);
                        psq.setChecktime(timestamp);
                    }else{
                        if(stuanswers.size() > 0){
                            coverted = 0;//上传的多张图片设为0未转换
                        }
                    }
                    psq.setAnswered(1);
                }
            }
        }

        Integer costtime = answerVo.getCosttime();
        if(costtime == null){
            costtime = 0;
        }
        psq.setCosttime(psq.getCosttime() + costtime);
        psq.setConverted(coverted);
    }

    /**
     * @Description 提交试卷
     * @Param [epsid]
     * @Return int
     * @Author wn
     * @Date 2022/7/28 16:59
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveCommitPaper(Integer epsid){
        EPaperStudent ps = ePaperStudentMapper.selectByPrimaryKey(epsid);
        if(ps == null || ps.getSstate() == 1){
            return;
        }
        EPaperStudent paperStudent = new EPaperStudent();
        paperStudent.setEpsid(epsid);
        paperStudent.setSstate(2);

        //更改试卷中学生未提交的试题设置为0分，标记为已批改
        ePaperStudentQuestionMapper.updateStuQuesitonNoCommit(epsid);
        //获取试卷中未批阅试题数量
        int noCheckNum = ePaperStudentQuestionMapper.getStuQuestionNoCheckNum(epsid);
        int checked = 0;
        if(noCheckNum == 0){
            checked = 1;//试卷中所有试题都批完后标识试卷已批改
        }
        paperStudent.setChecked(checked);
        ePaperStudentMapper.updateStudentPaper(paperStudent);//修改试卷信息
        chandleStudentQuestionNoconvertedPic(epsid,1);//处理试卷中多张图片未合并情况
    }

    //处理试卷中未已提交需要合并的图片,type1代表线上2代表扫码答题卡图片
    private void chandleStudentQuestionNoconvertedPic(Integer epsid,Integer type){
        List<EPaperStudentQuestion> psqlist = ePaperStudentQuestionMapper.listStuQuesitonStuAnswerPic(epsid);
        if(N_Utils.isListNotEmpty(psqlist)){
            for(EPaperStudentQuestion sq : psqlist){
                List<String> stuanswer = JSON.parseArray(sq.getStuanswer(),String.class);
                if(N_Utils.isListNotEmpty(stuanswer) && stuanswer.size()>1){
                    convertUtil.imgMerge(sq.getEpsqid(),stuanswer,type);
                }
            }
        }
    }

    /**
     * @Description 提交试卷、试题（网页端）
     * @Param [pswvo]
     * @Return void
     * @Author wn
     * @Date 2022/7/28 19:18
     **/
    public void saveCommitPaperForWeb(PaperStudentWebVo pswvo){
        List<EPaperStudentQuestion> sqlist = pswvo.getQuestions();//学生提交试卷中试题
        EPaperStudent ps = ePaperStudentMapper.selectByPrimaryKey(pswvo.getEpsid());
        //试卷不存在或者已提交
        if(ps == null || ps.getSstate() == 1){
            return;
        }
        //获取试卷中所有试题答案及分值
        List<PsqAnswerVo> anvolist = ePaperStudentQuestionMapper.listPaperQuestionsAnswer(pswvo.getEpsid());
        Integer checknum = 0;//记录试卷中试题批阅数量
        Double paperscore = 0D;//学生试卷得分
        for(EPaperStudentQuestion q : sqlist){

            //获取该试题正确答案
            PsqAnswerVo anvo = anvolist.stream().filter(a -> a.getEptqid().equals(q.getEptqid())).findFirst().orElse(null);

            setCommitQuestion(q,anvo);
            if(q.getChecked() == 2){
                checknum ++;
            }
            paperscore = ExamUtil.add(paperscore, q.getStuscore());
        }
        EPaperStudent paperStudent = new EPaperStudent();
        paperStudent.setEpsid(pswvo.getEpsid());
        paperStudent.setEndtime(N_Utils.getSecondTimestamp());
        paperStudent.setSstate(2);
        paperStudent.setStuscore(paperscore);
        int checked = 0;
        if(checknum == sqlist.size()){
            checked = 1;
        }
        paperStudent.setChecked(checked);

        ePaperStudentQuestionMapper.updateBatchStuQuestion(sqlist);
        ePaperStudentMapper.updateStudentPaper(paperStudent);

        //获取试卷中需要合并图片的试题
        chandleStudentQuestionNoconvertedPic(pswvo.getEpsid(),1);
    }

    /**
     * @Description 答题卡扫描时保存
     * @Param [pswvo]
     * @Return void
     * @Author wn
     * @Date 2022/8/3 9:17
     **/
    public void saveCommitPaperForScantron(PaperStudentWebVo pswvo) throws Exception{
        //更加学生姓名和准考证号找到对应学生
        Integer userid = userMapper.getUseridByExamno(pswvo.getExamno());
        if(!N_Utils.isTrueInteger(userid)){
            throw new Exception("考号有误");
        }
        //获取学生试卷
        EPaperStudent ps = ePaperStudentMapper.getPaperStudentByStuidAndEpid(pswvo.getEpid(),userid);
        pswvo.setEpsid(ps.getEpsid());

        //试卷不存在
        if(ps == null) {
            throw new Exception("学生试卷不存在");
        }

        if(pswvo.getSstate() == 3){
            //说明学生缺考更新状态
            EPaperStudent paperStudent = new EPaperStudent();
            paperStudent.setEpsid(pswvo.getEpsid());
            paperStudent.setStupic(JSON.toJSONString(pswvo.getStupic()));
            paperStudent.setSstate(3);
            if(!N_Utils.isEmptyInteger(pswvo.getSstate())){
                paperStudent.setHasbad(0);
            }else{
                paperStudent.setHasbad(1);
            }

            ePaperStudentMapper.updateStudentPaperState(paperStudent);
            ePaperStudentQuestionMapper.updateStuQuestionChecked(pswvo.getEpsid(),userid);
        }else{
            List<EPaperStudentQuestion> sqlist = pswvo.getQuestions();//学生提交试卷中试题

            //获取试卷中所有试题答案及分值
            List<PsqAnswerVo> anvolist = ePaperStudentQuestionMapper.listPaperQuestionsAnswer(pswvo.getEpsid());
            Integer checknum = 0;//记录试卷中试题批阅数量
            Double paperscore = 0D;//学生试卷得分
            for(EPaperStudentQuestion q : sqlist){
                q.setAnswertype(1);

                //获取该试题正确答案(根据题号区分)
                PsqAnswerVo anvo = anvolist.stream().filter(a -> a.getQn().equals(q.getQn())).findFirst().orElse(null);

                setCommitQuestion(q,anvo);
                if(q.getChecked() == 2){
                    checknum ++;
                }
                q.setEpsqid(anvo.getEpsqid());
                paperscore = ExamUtil.add(paperscore, q.getStuscore());
            }
            EPaperStudent paperStudent = new EPaperStudent();
            paperStudent.setEpsid(pswvo.getEpsid());
            paperStudent.setStupic(JSON.toJSONString(pswvo.getStupic()));
            paperStudent.setSstate(2);
            paperStudent.setStuscore(paperscore);
            int checked = 0;
            if(checknum == sqlist.size()){
                checked = 1;
            }
            paperStudent.setChecked(checked);

            ePaperStudentQuestionMapper.updateBatchStuQuestion(sqlist);
            ePaperStudentMapper.updateStudentPaper(paperStudent);
            //获取试卷中需要合并图片的试题
            chandleStudentQuestionNoconvertedPic(pswvo.getEpsid(),2);
        }
    }

}
